"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "createAlertingADCluster", {
  enumerable: true,
  get: function () {
    return _createAlertingADCluster.default;
  }
});
Object.defineProperty(exports, "createAlertingCluster", {
  enumerable: true,
  get: function () {
    return _createAlertingCluster.default;
  }
});

var _createAlertingCluster = _interopRequireDefault(require("./alerting/createAlertingCluster"));

var _createAlertingADCluster = _interopRequireDefault(require("./alerting/createAlertingADCluster"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
//# sourceMappingURL=data:application/json;charset=utf-8;base64,eyJ2ZXJzaW9uIjozLCJzb3VyY2VzIjpbImluZGV4LmpzIl0sIm5hbWVzIjpbXSwibWFwcGluZ3MiOiI7Ozs7Ozs7Ozs7Ozs7Ozs7OztBQUtBOztBQUNBIiwic291cmNlc0NvbnRlbnQiOlsiLypcbiAqIENvcHlyaWdodCBPcGVuU2VhcmNoIENvbnRyaWJ1dG9yc1xuICogU1BEWC1MaWNlbnNlLUlkZW50aWZpZXI6IEFwYWNoZS0yLjBcbiAqL1xuXG5pbXBvcnQgY3JlYXRlQWxlcnRpbmdDbHVzdGVyIGZyb20gJy4vYWxlcnRpbmcvY3JlYXRlQWxlcnRpbmdDbHVzdGVyJztcbmltcG9ydCBjcmVhdGVBbGVydGluZ0FEQ2x1c3RlciBmcm9tICcuL2FsZXJ0aW5nL2NyZWF0ZUFsZXJ0aW5nQURDbHVzdGVyJztcblxuZXhwb3J0IHsgY3JlYXRlQWxlcnRpbmdDbHVzdGVyLCBjcmVhdGVBbGVydGluZ0FEQ2x1c3RlciB9O1xuIl19